#ifndef flag_util_h
#define flag_util_h

/************************************************************************
 *
 * flag_util.h で定義可能なフラグ操作演算子:
 * ・フラグ積: &, &=
 * ・フラグ和: |, |=
 * ・排他的フラグ和: ^, |=
 * ・フラグ反転: ~
 *
 ***********************************************************************/

#include <type_traits>

#define define_flag_operators(NAME)                                                 \
inline NAME operator | (const NAME& x, const NAME& y) {                             \
  typedef std::underlying_type<NAME>::type base_type ;                              \
  return static_cast<NAME>(static_cast<base_type>(x) | static_cast<base_type>(y)) ; \
}                                                                                   \
inline NAME operator & (const NAME& x, const NAME& y) {                             \
  typedef std::underlying_type<NAME>::type base_type ;                              \
  return static_cast<NAME>(static_cast<base_type>(x) & static_cast<base_type>(y)) ; \
}                                                                                   \
inline NAME operator ^ (const NAME& x, const NAME& y) {                             \
  typedef std::underlying_type<NAME>::type base_type ;                              \
  return static_cast<NAME>(static_cast<base_type>(x) ^ static_cast<base_type>(y)) ; \
}                                                                                   \
inline NAME& operator |= (NAME& x, const NAME& y) {                                 \
  return x = x | y ;                                                                \
}                                                                                   \
inline NAME& operator &= (NAME& x, const NAME& y) {                                 \
  return x = x & y ;                                                                \
}                                                                                   \
inline NAME& operator ^= (NAME& x, const NAME& y) {                                 \
  return x = x ^ y ;                                                                \
}                                                                                   \
inline NAME operator ~ (const NAME& x) {                                            \
  typedef std::underlying_type<NAME>::type base_type ;                              \
  return static_cast<NAME>(~static_cast<base_type>(x)) ;                            \
}


#endif
