#ifndef gl__Context_h
#define gl__Context_h

#include "../noncopyable.h"
#include "Enabler.h"
#include "Matrix.h"
#include "TextureRenderer.h"
#include <vector>
#include <unordered_map>

namespace gl
{
  class GLObject ;
  class Texture ;
  class TextureObject ;

  /**
   * GLコンテキスト クラス。
   */
  class Context : noncopyable<Context>
  {
  protected :
    Context() ;
    virtual ~Context() ;

  public :
    /**
     * コンテキストの再描画を要求。
     */
    virtual void refresh() = 0 ;

    /**
     * エラーメッセージを取得。
     */
    const GLubyte *getErrorMessage() const { return gluGetString(glGetError()) ; }

#define GLCONTEXT_ENABLER(FlagName, name)  \
    Enabler<FlagName> name() {             \
      return Enabler<FlagName>();          \
    }                                      \
    Enabler<FlagName> name(bool enabled) { \
      return Enabler<FlagName>(enabled);   \
    }

    GLCONTEXT_ENABLER(GL_ALPHA_TEST, alpha_test) ;
    GLCONTEXT_ENABLER(GL_AUTO_NORMAL, auto_normal) ;
    GLCONTEXT_ENABLER(GL_BLEND, blend) ;
    GLCONTEXT_ENABLER(GL_COLOR_LOGIC_OP, color_logic_op) ;
    GLCONTEXT_ENABLER(GL_COLOR_MATERIAL, color_material) ;
    GLCONTEXT_ENABLER(GL_COLOR_SUM, color_sum) ;
    GLCONTEXT_ENABLER(GL_COLOR_TABLE, color_table) ;
    GLCONTEXT_ENABLER(GL_CONVOLUTION_1D, convolution_1d) ;
    GLCONTEXT_ENABLER(GL_CONVOLUTION_2D, convolution_2d) ;
    GLCONTEXT_ENABLER(GL_CULL_FACE, cull_face) ;
    GLCONTEXT_ENABLER(GL_DEPTH_TEST, depth_test) ;
    GLCONTEXT_ENABLER(GL_DITHER	, dither	) ;
    GLCONTEXT_ENABLER(GL_FOG, fog) ;
    GLCONTEXT_ENABLER(GL_HISTOGRAM, histogram) ;
    GLCONTEXT_ENABLER(GL_INDEX_LOGIC_OP, index_logic_op) ;
    GLCONTEXT_ENABLER(GL_LIGHTING, lighting) ;
    GLCONTEXT_ENABLER(GL_LINE_SMOOTH, line_smooth) ;
    GLCONTEXT_ENABLER(GL_LINE_STIPPLE, line_stipple) ;
    GLCONTEXT_ENABLER(GL_MAP1_COLOR_4, map1_color_4) ;
    GLCONTEXT_ENABLER(GL_MAP1_INDEX, map1_index) ;
    GLCONTEXT_ENABLER(GL_MAP1_NORMAL, map1_normal) ;
    GLCONTEXT_ENABLER(GL_MAP1_TEXTURE_COORD_1, map1_texture_coord_1) ;
    GLCONTEXT_ENABLER(GL_MAP1_TEXTURE_COORD_2, map1_texture_coord_2) ;
    GLCONTEXT_ENABLER(GL_MAP1_TEXTURE_COORD_3, map1_texture_coord_3) ;
    GLCONTEXT_ENABLER(GL_MAP1_TEXTURE_COORD_4, map1_texture_coord_4) ;
    GLCONTEXT_ENABLER(GL_MAP1_VERTEX_3, map1_vertex_3) ;
    GLCONTEXT_ENABLER(GL_MAP1_VERTEX_4, map1_vertex_4) ;
    GLCONTEXT_ENABLER(GL_MAP2_COLOR_4, map2_color_4) ;
    GLCONTEXT_ENABLER(GL_MAP2_INDEX, map2_index) ;
    GLCONTEXT_ENABLER(GL_MAP2_NORMAL, map2_normal) ;
    GLCONTEXT_ENABLER(GL_MAP2_TEXTURE_COORD_1, map2_texture_coord_1) ;
    GLCONTEXT_ENABLER(GL_MAP2_TEXTURE_COORD_2, map2_texture_coord_2) ;
    GLCONTEXT_ENABLER(GL_MAP2_TEXTURE_COORD_3, map2_texture_coord_3) ;
    GLCONTEXT_ENABLER(GL_MAP2_TEXTURE_COORD_4, map2_texture_coord_4) ;
    GLCONTEXT_ENABLER(GL_MAP2_VERTEX_3, map2_vertex_3) ;
    GLCONTEXT_ENABLER(GL_MAP2_VERTEX_4, map2_vertex_4) ;
    GLCONTEXT_ENABLER(GL_MINMAX, minmax) ;
    GLCONTEXT_ENABLER(GL_MULTISAMPLE, multisample) ;
    GLCONTEXT_ENABLER(GL_NORMALIZE, normalize) ;
    GLCONTEXT_ENABLER(GL_POINT_SMOOTH, point_smooth) ;
    GLCONTEXT_ENABLER(GL_POINT_SPRITE, point_sprite) ;
    GLCONTEXT_ENABLER(GL_POLYGON_OFFSET_FILL, polygon_offset_fill) ;
    GLCONTEXT_ENABLER(GL_POLYGON_OFFSET_LINE, polygon_offset_line) ;
    GLCONTEXT_ENABLER(GL_POLYGON_OFFSET_POINT, polygon_offset_point) ;
    GLCONTEXT_ENABLER(GL_POLYGON_SMOOTH, polygon_smooth) ;
    GLCONTEXT_ENABLER(GL_POLYGON_STIPPLE, polygon_stipple) ;
    GLCONTEXT_ENABLER(GL_POST_COLOR_MATRIX_COLOR_TABLE, post_color_matrix_color_table) ;
    GLCONTEXT_ENABLER(GL_POST_CONVOLUTION_COLOR_TABLE, post_convolution_color_table) ;
    GLCONTEXT_ENABLER(GL_RESCALE_NORMAL, rescale_normal) ;
    GLCONTEXT_ENABLER(GL_SAMPLE_ALPHA_TO_COVERAGE, sample_alpha_to_coverage) ;
    GLCONTEXT_ENABLER(GL_SAMPLE_ALPHA_TO_ONE, sample_alpha_to_one) ;
    GLCONTEXT_ENABLER(GL_SAMPLE_COVERAGE, sample_coverage) ;
    GLCONTEXT_ENABLER(GL_SEPARABLE_2D, separable_2d) ;
    GLCONTEXT_ENABLER(GL_SCISSOR_TEST, scissor_test) ;
    GLCONTEXT_ENABLER(GL_STENCIL_TEST, stencil_test) ;
    GLCONTEXT_ENABLER(GL_TEXTURE_1D, texture_1d) ;
    //GLCONTEXT_ENABLER(GL_TEXTURE_2D, texture_2d) ;  // Texture関連処理の中で自動的に使用／不使用を切り替え
    GLCONTEXT_ENABLER(GL_TEXTURE_3D, texture_3d) ;
    GLCONTEXT_ENABLER(GL_TEXTURE_CUBE_MAP, texture_cube_map) ;
    GLCONTEXT_ENABLER(GL_TEXTURE_GEN_Q, texture_gen_q) ;
    GLCONTEXT_ENABLER(GL_TEXTURE_GEN_R, texture_gen_r) ;
    GLCONTEXT_ENABLER(GL_TEXTURE_GEN_S, texture_gen_s) ;
    GLCONTEXT_ENABLER(GL_TEXTURE_GEN_T, texture_gen_t) ;
    GLCONTEXT_ENABLER(GL_VERTEX_PROGRAM_POINT_SIZE, vertex_program_point_size) ;
    GLCONTEXT_ENABLER(GL_VERTEX_PROGRAM_TWO_SIDE, vertex_program_two_side) ;
    
#undef GLCONTEXT_ENABLER

#define GLCONTEXT_MATRIX(ConstName, name)              \
    Matrix<ConstName, ConstName##_MATRIX> name() {     \
      return Matrix<ConstName, ConstName##_MATRIX>() ; \
    }

    GLCONTEXT_MATRIX(GL_PROJECTION, projection) ;
    GLCONTEXT_MATRIX(GL_MODELVIEW, modelview) ;
    GLCONTEXT_MATRIX(GL_TEXTURE, texture) ;
    GLCONTEXT_MATRIX(GL_COLOR, color) ;

#undef GLCONTEXT_MATRIX

    // TODO: 各種処理の実装

    // テクスチャ関連
  public :
    /**
     * テクスチャをbindするためのオブジェクトを取得。
     */
    TextureRenderer bind(Texture *tex) ;

  private :
    friend class Texture ;
    friend class TextureObject ;

    /**
     * テクスチャから、コンテキストごとのテクスチャオブジェクトを取得。
     */
    TextureObject *findObject(Texture *tex) ;

    /**
     * テクスチャから、コンテキストごとのテクスチャオブジェクトを作成。
     */
    TextureObject *createObject(Texture *tex) ;

    /**
     * テクスチャから、コンテキストごとのテクスチャオブジェクトを削除予約。
     */
    void eraseObject(Texture *tex) ;
    
  private :
    std::unordered_map<Texture *, TextureObject *> m_texture_map ;

    std::vector<GLObject *> m_erased_objects ;

    // コンテキストの管理
  public :
    /**
     * コンテキストに切り替わった際の処理。
     */
    void awake() ;

    template <typename Action>
    static void foreach(Action action)
    {
      for (Context *context : m_contexts) action(*context) ;
    }

  private :
    static std::vector<Context*> m_contexts ;
  } ;
}

#endif
