#include "Texture.h"
#include "Context.h"
#include "TextureObject.h"

gl::Texture::Texture(const Texture &another)
{
  m_target = another.m_target ;
  m_width  = another.m_width ;
  m_height = another.m_height ;
  m_internalFormat = another.m_internalFormat ;
  m_format = another.m_format ;
  m_type = another.m_type ;
  m_border = another.m_border ;

  if (nullptr == another.m_buffer) {
    m_buffer = nullptr ;
  }
  else {
    m_buffer_size = another.m_buffer_size ;
    m_buffer = new GLubyte[m_buffer_size] ;
    std::memcpy(m_buffer, another.m_buffer, m_buffer_size) ;
  }

  m_mag_filter = another.m_mag_filter ;
  m_min_filter = another.m_min_filter ;
  m_wrap_s = another.m_wrap_s ;
  m_wrap_t = another.m_wrap_t ;
  setBorderColor(another.m_border_color) ;
}

gl::Texture &gl::Texture::operator=(const gl::Texture &another)
{
  m_target = another.m_target ;
  m_width  = another.m_width ;
  m_height = another.m_height ;
  m_internalFormat = another.m_internalFormat ;
  m_format = another.m_format ;
  m_type = another.m_type ;
  m_border = another.m_border ;

  if (nullptr == another.m_buffer) {
    if (m_buffer) delete[] m_buffer ;
    m_buffer_size = 0 ;
    m_buffer = nullptr ;
  }
  else {
    if (!m_buffer) {
      // メモリを確保
      m_buffer_size = another.m_buffer_size ;
      m_buffer = new GLubyte[m_buffer_size] ;
    }
    else {
      if (m_buffer_size < another.m_buffer_size) {
        // 新サイズの方が大きいのでメモリを再確保
        m_buffer_size = another.m_buffer_size ;
        delete[] m_buffer ;
        m_buffer = new GLubyte[m_buffer_size] ;
      }
    }
    std::memcpy(m_buffer, another.m_buffer, another.m_buffer_size) ;
  }

  m_mag_filter = another.m_mag_filter ;
  m_min_filter = another.m_min_filter ;
  m_wrap_s = another.m_wrap_s ;
  m_wrap_t = another.m_wrap_t ;
  setBorderColor(another.m_border_color) ;

  return *this ;
}

void gl::Texture::notifyChange(ChangedTextureData data)
{
  Context::foreach([this, data](Context &context) {
    TextureObject *obj = context.findObject(this) ;
    if (obj) obj->notifyChange(data) ;
  }) ;
}

void gl::Texture::notifyDelete()
{
  Context::foreach([this](Context &context) {
    context.eraseObject(this) ;
  }) ;
}
