#include "TextureObject.h"

gl::TextureObject::TextureObject(Texture *tex)
  : m_texture(tex)
{
  glGenTextures(1, &m_id) ;
}

gl::TextureObject::~TextureObject()
{
  // コンテキスト破棄による削除でなければ、glDeleteTextures()を呼び出す。
  if (!m_suppress_gl_delete) glDeleteTextures(1, &m_id) ;
}

void gl::TextureObject::update()
{
  if (!m_change) return ;

  TextureRenderer renderer(this) ;

  if (!!(m_change & Texture::ChangedTextureData::Image)) {
    // 画像更新が必要
    glTexImage2D(static_cast<GLenum>(m_texture->m_target), 0, static_cast<GLenum>(m_texture->m_internalFormat),
                 m_texture->m_width, m_texture->m_height, m_texture->m_border,
                 static_cast<GLenum>(m_texture->m_format), static_cast<GLenum>(m_texture->m_type),
                 m_texture->m_buffer) ;
  }
  if (!!(m_change & Texture::ChangedTextureData::BorderColor)) {
    // 境界色更新が必要
    glTexParameterfv(static_cast<GLenum>(m_texture->m_target), GL_TEXTURE_BORDER_COLOR, m_texture->m_border_color) ;
  }
  if (!!(m_change & Texture::ChangedTextureData::MagFilter)) {
    // 拡大関数更新が必要
    glTexParameteri(static_cast<GLenum>(m_texture->m_target), GL_TEXTURE_MAG_FILTER, static_cast<GLenum>(m_texture->m_mag_filter)) ;
  }
  if (!!(m_change & Texture::ChangedTextureData::MinFilter)) {
    // 縮小関数更新が必要
    glTexParameteri(static_cast<GLenum>(m_texture->m_target), GL_TEXTURE_MIN_FILTER, static_cast<GLenum>(m_texture->m_min_filter)) ;
  }
  if (!!(m_change & Texture::ChangedTextureData::WrapS)) {
    // S方向のラップ更新が必要
    glTexParameteri(static_cast<GLenum>(m_texture->m_target), GL_TEXTURE_WRAP_S, static_cast<GLenum>(m_texture->m_wrap_s)) ;
  }
  if (!!(m_change & Texture::ChangedTextureData::WrapT)) {
    // T方向のラップ更新が必要
    glTexParameteri(static_cast<GLenum>(m_texture->m_target), GL_TEXTURE_WRAP_T, static_cast<GLenum>(m_texture->m_wrap_t)) ;
  }

  // 更新完了
  m_change = Texture::ChangedTextureData::None ;
}
