#ifndef gl__TextureObject_h
#define gl__TextureObject_h

#include "Context.h"
#include "Texture.h"
#include "GLObject.h"
#include "../noncopyable.h"

namespace gl
{
  /**
   * コンテキストごとのテクスチャオブジェクト。
   */
  class TextureObject : noncopyable<TextureObject>, public GLObject
  {
    friend class Texture ;

  public :
    /**
     * コンストラクタ。
     */
    TextureObject(Texture *tex) ;

    /**
     * デストラクタ。
     */
    virtual ~TextureObject() ;

    /**
     * テクスチャの変更点をコンテキストに反映。
     */
    void update() ;

    /**
     * 作成元のテクスチャを取得。
     */
    const Texture *getTexture() const { return m_texture ; }

    /**
     * コンテキストごとのテクスチャのID。
     */
    GLuint getID() const { return m_id ; }

  private :
    /**
     * テクスチャの変更点をマーク。
     */
    void notifyChange(Texture::ChangedTextureData data)
    {
      m_change |= data ;
    }

  private :
    GLuint m_id ;
    Texture *m_texture ;
    Texture::ChangedTextureData m_change = Texture::ChangedTextureData::All ;
  } ;
}

#endif
