#ifndef gl__TextureRenderer_h
#define gl__TextureRenderer_h

#include "../noncopyable.h"
#include "Texture.h"

namespace gl
{
  /**
   * 一時的にテクスチャを設定するためのクラス。
   */
  class TextureRenderer : noncopyable<TextureRenderer>
  {
    // gl::Contextクラスとgl::TextureObjectクラスでのみインスタンス化可能。
    friend class Context ;
    friend class TextureObject ;
    
  private :
    /**
     * テクスチャ有効化の開始。
     */
    TextureRenderer(TextureObject *obj) ;

  public :
    /**
     * ムーブコンストラクタ。
     */

    TextureRenderer(TextureRenderer &&another)
      : m_target(another.m_target), m_last_id(another.m_last_id), m_moved(another.m_moved)
    {
      // 移動元を移動済みとする
      another.m_moved = true ;
    }

    /**
     * デストラクタ。
     */
    ~TextureRenderer()
    {
      // 移動済みでない場合は元のテクスチャに戻す
      if (!m_moved) glBindTexture(m_target, m_last_id) ;
    }

    /**
     * 頂点にテクスチャ座標値を設定。
     */
    void coord(GLfloat s, GLfloat t) { glTexCoord2f(s, t) ; }
    void coord(GLdouble s, GLdouble t) { glTexCoord2d(s, t) ; }
    void coord(GLint s, GLint t) { glTexCoord2i(s, t) ; }
    void coord(GLshort s, GLshort t) { glTexCoord2s(s, t) ; }

  private :
    GLenum m_target ;
    GLint m_last_id ;
    bool m_moved ;
  } ;
}

#endif
