#ifndef gl__gl_types_h
#define gl__gl_types_h

#include "../glall.h"

namespace gl
{
  /**
   * OpenGLの型名→型を取得するクラス。
   *
   * 型名に対応する定数はtype_info<T>::valueで取得できる。
   *
   * @param T データ型。
   */
  template <class T> struct type_info {} ;
  template<> struct type_info<GLbyte>
  {
    static const GLenum value = GL_BYTE ;
  } ;
  template<> struct type_info<GLubyte>
  {
    static const GLenum value = GL_UNSIGNED_BYTE ;
  } ;
  template<> struct type_info<GLshort>
  {
    static const GLenum value = GL_SHORT ;
  } ;
  template<> struct type_info<GLushort>
  {
    static const GLenum value = GL_UNSIGNED_SHORT ;
  } ;
  template<> struct type_info<GLint>
  {
    static const GLenum value = GL_INT ;
  } ;
  template<> struct type_info<GLuint>
  {
    static const GLenum value = GL_UNSIGNED_INT ;
  } ;
  template<> struct type_info<GLfloat>
  {
    static const GLenum value = GL_FLOAT ;
  } ;
  template<> struct type_info<GLdouble>
  {
    static const GLenum value = GL_DOUBLE ;
  } ;
  template<> struct type_info<GLbute[2]>
  {
    static const GLenum value = GL_2_BYTES ;
  } ;
  template<> struct type_info<GLbute[3]>
  {
    static const GLenum value = GL_3_BYTES ;
  } ;
  template<> struct type_info<GLbute[4]>
  {
    static const GLenum value = GL_4_BYTES ;
  } ;

  // 以下、glGetをオーバーロードで置き換える関数群
  inline static void getv(GLenum pname, GLint *ptr)
  {
    glGetIntegerv(pname, ptr) ;
  }
  inline static void getv(GLenum pname, GLuint index, GLint *ptr)
  {
    glGetIntegeri_v(pname, index, ptr) ;
  }
  
  inline static void getv(GLenum pname, GLuint *ptr)
  {
    glGetIntegerv(pname, reinterpret_cast<GLint*>(ptr)) ;
  }
  inline static void getv(GLenum pname, GLuint index, GLuint *ptr)
  {
    glGetIntegeri_v(pname, index, reinterpret_cast<GLint*>(ptr)) ;
  }
  
  inline static void getv(GLenum pname, GLfloat *ptr)
  {
    glGetFloatv(pname, ptr) ;
  }
  inline static void getv(GLenum pname, GLuint index, GLfloat *ptr)
  {
    glGetFloati_v(pname, index, ptr) ;
  }
  inline static void getv(GLenum pname, GLdouble *ptr)
  {
    glGetFloatv(pname, ptr) ;
  }
  inline static void getv(GLenum pname, GLuint index, GLdouble *ptr)
  {
    glGetFloati_v(pname, index, ptr) ;
  }
}

#endif
