#include "glall.h"
#include "glut/Window.h"
#include "gl/Enabler.h"
#include "gl/Texture.h"

/**
 * glut::Windowを継承してウィンドウを作る
 */
class Window : public glut::Window
{
public :
  Window(const char *name)
    : glut::Window(name), m_texture(gl::TextureType::Normal)
  {
    GLuint data[] = {
      0xFFFF0000, 0xFFFFFFFF,
      0xFF0000FF, 0xFF00FF00,
    } ;
    m_texture.setBuffer<>(2, 2, (GLubyte*)data) ;
  }

private :
  gl::Texture m_texture ;

protected :
  virtual void reshape(gl::Context &context, int width, int height) override
  {
    glViewport(0, 0, width, height) ;
  }

  virtual void display(gl::Context &context) override
  {
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT) ;
    glEnable(GL_DEPTH_TEST) ;

    {
      // テクスチャを有効化
      auto binder = context.bind(&m_texture) ;

      // 奥行きを持たせた四角形を表示
      glBegin(GL_TRIANGLE_FAN) ;
      binder.coord(0, 0) ;
      glColor3ub(255, 255, 255) ;
      glVertex3d(-1, -1, -1) ;
      binder.coord(0, 2) ;
      glVertex3d(-1, +1, -1) ;
      glColor3ub(128, 128, 128) ;
      binder.coord(2, 2) ;
      glVertex3d(+1, +1, +1) ;
      binder.coord(2, 0) ;
      glVertex3d(+1, -1, +1) ;
      glEnd() ;
    }

    // 一時的にX方向を1/2にして表示
    {
      auto modelview = context.modelview() ;
      auto m = Matrix<4,4,float>(1) ;
      m.at(0, 0) = 0.5 ;
      modelview *= m ;

      // デプステストを無効化して、Z=0に四角形を表示
      glDisable(GL_DEPTH_TEST) ;
      drawRect(context, -0.8, +0.8, +0.3, +0.8) ;
      fillRect(context, -0.8, +0.8, +0.3, +0.8) ;
    }

    // デプステストを有効化して、Z=0に四角形を表示
    glEnable(GL_DEPTH_TEST) ;
    drawRect(context, -0.8, +0.8, -0.8, -0.3) ;
    fillRect(context, -0.8, +0.8, -0.8, -0.3) ;

    glFinish() ;
  }

private :
  void drawRect(gl::Context &context, double l, double r, double b, double t)
  {
    auto x = context.depth_test(false);

    glLineWidth(3) ;
    glColor3ub(255, 0, 0) ;
    glBegin(GL_LINE_LOOP) ;
    glVertex3d(l, b, 0) ;
    glVertex3d(l, t, 0) ;
    glVertex3d(r, t, 0) ;
    glVertex3d(r, b, 0) ;
    glEnd() ;
  }
  
  void fillRect(gl::Context &context, double l, double r, double b, double t)
  {
    glColor3ub(255, 0, 255) ;
    glBegin(GL_TRIANGLE_FAN) ;
    glVertex3d(l, b, 0) ;
    glVertex3d(l, t, 0) ;
    glVertex3d(r, t, 0) ;
    glVertex3d(r, b, 0) ;
    glEnd() ;
  }
} ;

int main(int argc, char *argv[])
{
  glutInit(&argc, argv) ;
  glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH);

  Window w("OpenGLをC++11でラップしてみんとてするなり") ;

  glutMainLoop() ;

  return 0;
}

