#ifndef noncopyable_h
#define noncopyable_h

/**
 * コピー不能なクラスを作るための基底クラス。
 *
 * @param DerivedClass コピー不能となるクラス自身の型（class Hoge : noncopyable<Hoge> {...};）。
 */
template <class DerivedClass>
class noncopyable
{
protected :
  /**
   * noncopyableクラスは必ず継承しなければならないため、protectedとする。
   */
  noncopyable() {}
  ~noncopyable() {}

  /**
   * コピーコンストラクタを削除。
   */
  noncopyable(const noncopyable &) = delete ;

  /**
   * ムーブコンストラクタは許可。
   */
  noncopyable(noncopyable&&) {}

private :
  /**
   * コピー代入は不可。
   */
  DerivedClass &operator=(const DerivedClass&) = delete ;
} ;

#endif
